package insalyon.tracecourbe;

import android.view.View;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.os.Handler;
import android.util.AttributeSet;

public class GraphView extends View implements Nextable {
	private int yMax=10;
	private int xMax=10;
	private List<Integer> values;
	final Handler mHandler = new Handler();
    private Boolean alreadyWaiting=false;


    private Paint defaultBrush=new Paint();
    private Paint axisBrush=new Paint();
    private Paint labelsBrush=new Paint();

	// Constructeurs et initiateurs
	public GraphView(Context context) {
		super(context);
		setBackgroundColor(Color.DKGRAY);
	}
	public GraphView(Context context, AttributeSet attrs) {
		super(context, attrs);
		setBackgroundColor(Color.DKGRAY);
	}
	public GraphView(Context context, AttributeSet attrs, int defStyle) {
		super(context, attrs, defStyle);
		setBackgroundColor(Color.DKGRAY);
	}

	private void initBrushes(){
		defaultBrush.setColor(Color.WHITE);
		defaultBrush.setAlpha(255);
		defaultBrush.setAntiAlias(true);

		axisBrush.setColor(Color.CYAN);
		axisBrush.setAlpha(255);
		axisBrush.setAntiAlias(true);

		labelsBrush.setColor(Color.RED);
		labelsBrush.setAlpha(255);
		labelsBrush.setAntiAlias(true);
		labelsBrush.setTextSkewX((float)-0.25);	
	}

	
	public void initView(int yMax, int xMax){
		this.yMax=yMax;
		this.xMax=xMax;
		values = new ArrayList<Integer>(xMax);
		initBrushes();
	}
	
	public void initView(int yMax){
		this.yMax=yMax;
		values = new LinkedList<Integer>();
		initBrushes();
	}
	
	
	int v=10;
	public void next(){
		v=v+(int)(2*Math.random());
		add(v);
	}
	
	
	// Gestion des données
	private void add(int y){
		synchronized (values){
			values.add(y);
			if(values.size()>xMax){xMax=values.size()+30;}
			if(y>yMax){yMax=y;}
			flush();
		}
	}

	
	// Conversions
	private int toScreenValueNumber(int i){
		return toScreenY(values.get(i));
	}

	private int toScreenY(int y){
		return getHeight()-getHeight()*y/yMax;
	}

	private int toScreenX(int x){
		return getWidth()*x/xMax;
	}



	
	
	// Affichage
	private void flush(){
		synchronized(alreadyWaiting){
			if(!alreadyWaiting){
				mHandler.post(runnableFlush);
				alreadyWaiting=true;
			}
		}
	}

	
	final Runnable runnableFlush = new Runnable() {
        public void run() {
        	invalidate();
        }
    };

    
	@Override
	protected void onDraw(Canvas canvas) {
		super.onDraw(canvas);
		if(values!=null) {
			synchronized (values){
				int n=values.size();
				int h=getHeight();
				int h33=h/33;
				labelsBrush.setTextSize(6+(h/40));

				if(n>=2) {
					for(int i=0;i<n-1 ;i++){
						canvas.drawLine(toScreenX(i), toScreenValueNumber(i), toScreenX(i+1), toScreenValueNumber(i+1), defaultBrush);

						// Chevrons
						if((i+1)%10==0){
							canvas.drawLine(toScreenX(i+1), h, toScreenX(i+1), h-h33, axisBrush);
						}
					}

					labelsBrush.setTextAlign(Paint.Align.CENTER);
					canvas.drawText("y : "+values.get(n-1)+" / hght="+getHeight(),getWidth()/2, getHeight()/2,labelsBrush);
				}
				canvas.drawLine(0, h, getWidth(), h, axisBrush);
				canvas.drawLine(0, 0, 0, h, axisBrush);

			}

			alreadyWaiting=false;
		}
	}
}
